/*******************************************************************************
  File: raise.h
  Copyright (c) 1999 Bleading Edge Software Technologies, Inc.

  This Software is distributed under the open source, free software model.
  Permission to use, modify, and distribute this Software, including
  incorporating it into proprietary software, is granted without fee, 
  provided this copyright notice appear in all copies of the code.
  You are under no obligation to distribute source code which is derived from
  or uses this Software. You may not do anything however, to prevent the 
  continued distribution of this Software under an open source distribution
  model. This includes, but is not limited to, such things as copyrighting
  or claiming authorship of the Software.
  This Software is distributed in the hope that it will be useful, but it is
  provide "as is", WITHOUT ANY WARRANTY; without even the implied warranty
  of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

//$-----------------------------------------------------------------------------
$Header: $
  Author: Jack W. Reeves

//!-----------------------------------------------------------------------------
Description:
  Provides the 'raise' template which can be used to throw an exception, after
  first invoking any user supplied "raise_handler" (similar to the standard
  "terminate_handler" or "unexpected_handler").

  Adds a number of helper functions for dealing with exceptions.
  
//#-----------------------------------------------------------------------------
Notes:

//%****************************************************************************/
#ifndef UTIL_RAISE_H
#define UTIL_RAISE_H

// Platform
#include "platform.h"

// Standard C++ headers
#include <exception>
#include <string>

//
// raise handler
//
typedef void (*raise_handler)(const std::exception&);

raise_handler set_raise_handler(raise_handler handler);

//
// Helper function for creating standard what strings
//
std::string	make_what_str(const std::string& why, const std::string& where = "");
std::string	make_what_str(const std::string& what, const std::string& why, const std::string& where);

//
// Helper template for throwing an exception (
//	after checking the raise_handler).
//
template <typename Exception>
void raise(const Exception& ex)
{
    raise_handler h = set_raise_handler(null);
    if (h) {
        set_raise_handler(h);
        (*h)(ex);
    }
    throw ex;
}

// Variations on a theme
template <typename Exception>
void raise(const std::string& why, const std::string& where = "")
{
	raise(Exception(make_what_str(why, where)));
}

template <typename Exception>
void raise(const std::string& what, const std::string& why, const std::string& where)
{
	raise(Exception(make_what_str(what, why, where)));
}

#endif
